<?php
global $ADEI_ROOTDIR;
require_once($ADEI_ROOTDIR . "/classes/jpgraph.php");

abstract class AggregationMethods
{
    const SkipMissing = "skip";
    const UseLast = "uselast";

    static function GetSelectOptions()
    {
        return array(
            array("value" => self::SkipMissing, "label" => "Skip Where Missing"),
            array("value" => self::UseLast, "label" => "Use Last Available Value"),
        );
    }
}

class CALCView extends VIEW
{
    var $raw_data;
    
    const small_window_threshold = 400;
    function __construct(REQUEST $req = NULL, $options)
    {

        $this->raw_data = $this->GetOption('raw_data', false);
        parent::__construct($req, $options);
    }

    function GetOptions()
    {
        $chan = $this->GetDataVariants(array("x", "y"));

        return array(
            array("select" => array("label" => _("X"), "id" => "x", "options" => $chan[0])),
            array("xml" => "<br/>"),
            array("select" => array("label" => _("Y"), "id" => "y", "options" => $chan[1])),
            array("xml" => "<br/>"),
            array("select" => array("label" => _("Aggregation Method"), "id" => "channel_aggregation", "options" => AggregationMethods::GetSelectOptions())),
            array("xml" => "<br/>"),
            array("input" => array("label" => _("Math Function"), "id" => "math_func", "type" => "text", "size" => "8", "value" => "")),
            array("xml" => "<br/>"),
        );
    }

    function GetView()
    {
        $req = $this->req->CreateDataRequest();
        $agg = $req->GetProp("view_channel_aggregation", false);
        $this->math_func = $req->GetProp("view_math_func", false);

        if (!$this->math_func) {
            return array("info" => "Math Function is empty!");
        }

        $x = $req->GetProp("view_x", false);
        $y = $req->GetProp("view_y", false);

        if (!$x) throw new ADEIException(translate("Parameters view_x are not set"));

        if (!$y) throw new ADEIException(translate("Parameters view_y are not set"));

        $func = dsMathCreateFunction('$x,$y', $this->math_func);
        $res = $this->GetData(array("x", "y"), NULL, ($this->raw_data?VIEW::GET_RAW_DATA:0));

        $total = 0;
	$missing = 0;
        $data = array();

        switch ($agg) {
            case AggregationMethods::SkipMissing:
                foreach ($res['data'] as $t => $v) {
		    $cond = is_numeric($v[0]) && is_numeric($v[1]);
                    if ($cond) {
                        $point = $func($v[0], $v[1]);
                        $total++;
                        $data[$t] = $point;
                    } else {
			$missing++;
		    }
                }
                break;
            case AggregationMethods::UseLast:
                $raw_data = $res['data'];
                $raw_data_keys = array_keys($raw_data);
		$raw_data_values = array_values($raw_data);
                for ($i = 0; $i < sizeof($raw_data_values); $i++) {
                    $key = $raw_data_keys[$i];
                    $v = &$raw_data_values[$i];
                    
		    if(!is_numeric($v[0]) || !is_numeric($v[1])) {
			$missing++;
		    }

		    if (!is_numeric($v[0]) && $i > 0) {
                        $v[0] = $raw_data_values[$i - 1][0];
                    }

                    if (!is_numeric($v[1]) && $i > 0) {
                        $v[1] = $raw_data_values[$i - 1][1];
                    }

                    $point = $func($v[0], $v[1]);
                    $total++;
                    $data[$key] = $point;
                }
                break;
        }

	ksort($data);
	$el_parent_x = '';
	$el_x = '';
        $el_parent_y = '';
	$el_y = '';
        $arr = $this->GetOptions();
        foreach ($arr[0]['select']['options'] as $x_key => $x_lv) {
	    $el = $arr[0]['select']['options'][$x_key];
            if ($el["disabled"] == 1) {
                $el_parent_x = $el['label'];
	    } elseif ($el['value'] == $x) {
		$el_x = $el['label'];
		break;
	    }
        }

        foreach ($arr[2]['select']['options'] as $y_key => $y_lv) {
	    $el = $arr[2]['select']['options'][$y_key];
            if ($el["disabled"] == 1){
                $el_parent_y = $el['label'];
            } elseif ($el['value'] == $y) {
		$el_y = $el["label"];
		break;
            }
        }

	$info = array(
            array("title" => _("Data Points"), "value" => "$total (of $this->max_points)"),
	    array("title" => _("Formula"), "value" => "$this->math_func"),
	    array("title" => _("x"), "value" => "$el_parent_x - $el_x"),
	    array("title" => _("y"), "value" => "$el_parent_y - $el_y"),
	    array("title" => _("Missing or Aggregated Data Points"), "value" => "$missing"),
 	);

        if (sizeof($data) == 0) {
            return array("info" => $info);
        }

	$from = date("D, d M Y H:i:s", key($data));
	end($data);
	$to = date("D, d M Y H:i:s", key($data));
	reset($data);
	$info = array_merge($info, array(
		array("title" => _("From"), "value" => "$from"),
        	array("title" => _("To"), "value" => "$to"))
	);

        $view = $this->GetGraphView($data, $req);
	$view["info"] = $info;
        return $view;
    }

    private function GetTicks($width, $length, $threshold) {
	$majTicks = ceil($length / $threshold);
	if($width < self::small_window_threshold || $majTicks < 20) {
	    return floor($length/4);
	}

	return floor($majTicks/10);
    }

    private function GetGraphView($data, $req)
    {
        global $TMP_PATH;
	global $GRAPH_SUBSECOND_THRESHOLD;
        $tmp_file = ADEI::GetTmpFile();
	
	$x = $req->GetProp("view_x", false);
        $y = $req->GetProp("view_y", false);

        if ($this->object) {
            $width = $req->GetProp($this->object . "_width", $this->min_width + 20) - 20;
            if ($width < $this->min_width) $width = $this->min_width;
            $height = $width - 40;
        } else {
            $width = $req->GetProp("page_width", $this->min_width + 5) - 5;
            $height = $req->GetProp("page_height", $this->min_height);
            if ($width < $this->min_width) $width = $this->min_width;
            if ($height < $this->min_height) $height = $this->min_height;
        }

        $graph = new Graph($width, $height);
        $graph->SetScale('datlin');
	$graph->xaxis->SetPos('min');
	$graph->xaxis->SetLabelAngle(90);
        $graph->xaxis->title->SetFont(FF_ARIAL, FS_BOLD);
        $graph->yaxis->title->SetFont(FF_ARIAL, FS_BOLD);
        $graph->yaxis->SetTitle("$this->math_func");

        $ydata = array_values($data);
        $xdata = array_keys($data);

	$max_val = max($ydata); 
        $yaxis_margin = strlen("$max_val") * 5;
        $graph->img->SetMargin(20 + $yaxis_margin, 10, 10, 60);
        $graph->yaxis->SetTitleMargin($yaxis_margin);

        $length = dsMathPreciseSubstract($xdata[sizeof($xdata) - 1], $xdata[0]);
	if ($length > 315360000) { // 10 years
            $graph->xaxis->scale->SetDateFormat('Y');
            $graph->xaxis->SetTextLabelInterval(1);
	    $graph->xaxis->scale->ticks->Set($this->GetTicks($width, $length, 315360000), 315360000);
        } elseif ($length > 31104000) { // 1 year
            $graph->xaxis->scale->SetDateFormat('M, Y');
            $graph->xaxis->SetTextLabelInterval(2);
	    $graph->xaxis->scale->ticks->Set($this->GetTicks($width, $length, 31104000), 3104000);
        } elseif ($length > 1036800) { // 12 days
            $graph->xaxis->scale->SetDateFormat('M d');
            $graph->xaxis->SetTextLabelInterval(3);
	    $graph->xaxis->scale->ticks->Set($this->GetTicks($width, $length, 1036800), 3600*24);
        } elseif ($length > 86400) { // 1 day
            $graph->xaxis->scale->SetDateFormat('M d, H:i');
            $graph->xaxis->SetTextLabelInterval(4);
	    $graph->xaxis->scale->ticks->Set($this->GetTicks($width, $length, 86400), 3600 * 4);
        } elseif ($length > 14400) { // 4 hours
            $graph->xaxis->scale->SetDateFormat('H:i');
            $graph->xaxis->SetTextLabelInterval(4);
            $graph->xaxis->scale->ticks->Set($this->GetTicks($width, $length, 14400), 1800);
        } elseif ($length > $GRAPH_SUBSECOND_THRESHOLD) {
            $graph->xaxis->scale->SetDateFormat('H:i:s');
            $graph->xaxis->SetTextLabelInterval(1);
            $majTicks = $this->GetTicks($width, $length, $GRAPH_SUBSECOND_THRESHOLD);
	    $graph->xaxis->scale->ticks->Set($majTicks, $majTicks/10);
        } else {
            $from = $xdata[0];
            $ifrom = floor($from);
            if (is_float($from)) {
                $rfrom = substr(printf("%.9F", $from - $ifrom), 2);
            } else {
                $pos = strpos($from, ".");
                $rfrom = $pos === false ? 0 : substr($from, $pos + 1);
            }

            if ($rfrom) {
                $date_title .= "." . $rfrom;
                $rfrom = "0.$rfrom";
            }

            if ($length > $this->width / 1000) {
                $coef = 1000;
            } elseif ($length > $this->width / 1000000) {
                $coef = 1000000;
            } else {
                $coef = 1000000000;
            }

            $reallength = floor($length * $coef);
            $rem = ($reallength + 10) % 1000;
            $ilength = $rem < 20 ? $reallength - ($rem - 10) : $reallength;

            $first_num = substr($ilength, 0, 1);
            $second_num = substr($ilength, 1, 1);
            if ($second_num > 4) {
                if ((++$first_num) == 10) {
                    $istep = "1" . str_repeat("0", strlen($ilength) - 1);
                } else {
                    $istep = $first_num . str_repeat("0", strlen($ilength) - 2);
                }
            } else {
                $istep = $first_num . str_repeat("0", strlen($ilength) - 2);
            }

            $ticks = array($istep, $istep / 4);
            $graph->xaxis->scale->ticks->Set($ticks[0], $ticks[1]);
	    $ticks = floor(($ilength + 1) / $istep);
            if ($width < 350) {
                if ($width < 70) $lpt = $ticks;
                else $lpt = ceil($ticks / ($width / 70));
            } else  {
		$lpt = round($ticks / 5);
	    }

            $graph->xaxis->SetTextLabelInterval($lpt);
            $graph->xaxis->scale->SetDateFormat(false);
        }

        $lineplot = new LinePlot($ydata, $xdata);

        $graph->Add($lineplot);
        $graph->Stroke("$TMP_PATH/$tmp_file");

        return array(
            "img" => array("id" => $tmp_file)
        );
    }
};
